@@ -1,5 +1,5 @@
 __all__ = ['commands', 'datastore', 'formatting', 'launch', 'parameters',
            'programs', 'projects', 'records', 'recordstore', 'versioncontrol',
-           'dependency_finder', 'external', 'web']
+           'dependency_finder', 'external', 'web', 'decorators']
 
-__version__ = "0.2"    
\ No newline at end of file
+__version__ = "0.2.1"    
