@@ -18,6 +18,7 @@ get_repository()        - return a GitRepository object for a given URL.
 
 import git
 import os
+import shutil
 
 from base import Repository, WorkingCopy, VersionControlError
 
@@ -58,7 +59,10 @@ class GitWorkingCopy(WorkingCopy):
 
     def current_version(self):
         head = self.repository._repository.head
-        return head.commit.sha
+        try:
+            return head.commit.hexsha
+        except AttributeError:
+            return head.commit.sha
     
     def use_version(self, version):
         assert not self.has_changed()
@@ -80,6 +84,15 @@ class GitWorkingCopy(WorkingCopy):
         return g.diff('HEAD')
         
 
+def move_contents(src, dst):
+    for file in os.listdir(src):
+        src_path = os.path.join(src, file)
+        if os.path.isdir(src_path):
+            shutil.copytree(src_path, os.path.join(dst, file))    
+        else:
+            shutil.copy2(src_path, dst)
+    shutil.rmtree(src)
+
 class GitRepository(Repository):
     
     def __init__(self, url):
@@ -96,7 +109,9 @@ class GitRepository(Repository):
             # already have a repository in the working directory
             pass
         else:
-            g.clone(self.url, path)           
+            tmpdir = os.path.join(path, "tmp_git")
+            g.clone(self.url, tmpdir)
+            move_contents(tmpdir, path)
         self._repository = git.Repo(path)
         self.working_copy = GitWorkingCopy(path, repository=self)
     
