@@ -17,8 +17,14 @@ get_repository()        - return a MercurialRepository object for a given URL.
 """
 
 from mercurial import hg, ui, patch
-from mercurial.repo import RepoError
-from mercurial.dispatch import _findrepo # this has moved in more recent versions - need try..except ImportError
+try:
+    from mercurial.error import RepoError
+except ImportError:
+    from mercurial.repo import RepoError
+try:
+    from mercurial.cmdutil import findrepo
+except ImportError:
+    from mercurial.dispatch import _findrepo as findrepo
 import os
 import binascii
 from base import VersionControlError
@@ -28,7 +34,7 @@ from base import Repository, WorkingCopy
 
 def may_have_working_copy(path=None):
     path = path or os.getcwd()
-    return bool(_findrepo(path))
+    return bool(findrepo(path))
 
 def get_working_copy(path=None):
     if may_have_working_copy(path):
